package com.jqtx.windows.component.impl;

import cn.hutool.core.bean.BeanUtil;
import com.jqtx.windows.component.LoanPlanComponent;
import com.jqtx.windows.component.model.LoanPlanModel;
import com.jqtx.windows.infrastructure.feign.enums.DebtInfoStatusEnum;
import com.jqtx.windows.infrastructure.feign.enums.LoanStatusEnum;
import com.jqtx.windows.repository.entity.WindowsPlan;
import com.jqtx.windows.repository.entity.WindowsPlanExample;
import com.jqtx.windows.repository.mapper.WindowsPlanMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.Date;
import java.util.List;

@Service
public class LoanPlanComponentImpl implements LoanPlanComponent {

    @Autowired
    private WindowsPlanMapper windowsPlanMapper;

    @Override
    public int insert(LoanPlanModel loanPlanModel) {
        WindowsPlan windowsPlan = BeanUtil.toBean(loanPlanModel, WindowsPlan.class);
        int i = windowsPlanMapper.insertSelective(windowsPlan);
        return i;
    }

    @Override
    public List<LoanPlanModel> getByLoan(String loanNo) {
        WindowsPlanExample example = new WindowsPlanExample();
        WindowsPlanExample.Criteria criteria = example.createCriteria();
        criteria.andLoanNoEqualTo(loanNo);
        List<LoanPlanModel> loanPlanModels = BeanUtil.copyToList(windowsPlanMapper.selectByExample(example), LoanPlanModel.class);
        return loanPlanModels;
    }

    @Override
    public int updateByLoanPlanModel(LoanPlanModel loanPlanModel) {
        WindowsPlanExample example = new WindowsPlanExample();
        WindowsPlanExample.Criteria criteria = example.createCriteria();
        criteria.andIdEqualTo(loanPlanModel.getId());
        WindowsPlan windowsLoan = BeanUtil.toBean(loanPlanModel, WindowsPlan.class);
        int i = windowsPlanMapper.updateByExampleSelective(windowsLoan, example);
        return i;
    }

    @Override
    public LoanPlanModel getPlanNoByLoan(String loanNo, String planIndex) {
        WindowsPlanExample example = new WindowsPlanExample();
        WindowsPlanExample.Criteria criteria = example.createCriteria();
        criteria.andLoanNoEqualTo(loanNo);
        criteria.andPlanIndexEqualTo(Integer.parseInt(planIndex));
        List<WindowsPlan> windowsPlans = windowsPlanMapper.selectByExample(example);
        return windowsPlans.stream().map(m -> BeanUtil.toBean(m, LoanPlanModel.class)).findFirst().orElse(null);
    }

    @Override
    public long countLoanPlanByStatus(String loanNo) {
        WindowsPlanExample example = new WindowsPlanExample();
        WindowsPlanExample.Criteria criteria = example.createCriteria();
        criteria.andLoanNoEqualTo(loanNo);
        criteria.andStatusEqualTo(DebtInfoStatusEnum.BACKING.getCode());
        long backing = windowsPlanMapper.countByExample(example);

        criteria.andStatusEqualTo(DebtInfoStatusEnum.OVERDUED.getCode());
        long overdued = windowsPlanMapper.countByExample(example);
        return backing + overdued;
    }

    @Override
    public List<LoanPlanModel> getWillOverdueList() {
        WindowsPlanExample example = new WindowsPlanExample();
        WindowsPlanExample.Criteria criteria = example.createCriteria();
        criteria.andPlanDateLessThan(LocalDate.now());
        criteria.andStatusEqualTo(LoanStatusEnum.BACKING.getCode());
        List<LoanPlanModel> loanPlanModels = BeanUtil.copyToList(windowsPlanMapper.selectByExample(example), LoanPlanModel.class);
        return loanPlanModels;
    }

    @Override
    public LoanPlanModel getPlanNoByPlanNo(String planNo) {
        WindowsPlanExample example = new WindowsPlanExample();
        WindowsPlanExample.Criteria criteria = example.createCriteria();
        criteria.andPlanNoEqualTo(planNo);
        List<WindowsPlan> windowsPlans = windowsPlanMapper.selectByExample(example);
        return windowsPlans.stream().map(m -> BeanUtil.toBean(m, LoanPlanModel.class)).findFirst().orElse(null);
    }


    @Override
    public BigDecimal countBackedAmtByLoanNo(String loanNo) {
        WindowsPlanExample example = new WindowsPlanExample();
        WindowsPlanExample.Criteria criteria = example.createCriteria();
        criteria.andLoanNoEqualTo(loanNo);
        List<WindowsPlan> windowsPlans = windowsPlanMapper.selectByExample(example);
        BigDecimal backedAmt = BigDecimal.ZERO;
        for (WindowsPlan windowsPlan : windowsPlans) {
            backedAmt = backedAmt.add(windowsPlan.getBackedBen()).add(windowsPlan.getBackedXi()).add(windowsPlan.getBackedFu())
                    .add(windowsPlan.getBackedFa()).add(windowsPlan.getBackedBurden()).add(windowsPlan.getBackedBreak());
        }
        return backedAmt;
    }
}
